require 'spec_helper'

RSpec.describe Frigate::Form::Base do
	let(:params) { { email: 'z1@zzz.zz', user_profile: { first_name: 'John', last_name: 'Doe', skype: 'johndoe' } } }
	let(:tree_params) { params.tap { |_p| _p[:user_profile][:user_profile_passport] = { number: 'KL1234', country: 'Ukraine', city: 'Kiev' } } }
	let(:user_model) { User.new.tap { |user| user.build_user_profile } }
	let(:user_model_tree) { User.new.tap { |user| user.build_user_profile; user.profile.build_user_profile_passport } }

	before(:example) do
		@user_model = user_model
		@user_model_tree = user_model_tree
		@user_form = UserForm.new(@user_model)
		@user_form_custom = UserFormCustom.new(@user_model)
		@user_form_tree = UserFormTree.new(@user_model_tree)
	end

	context '#valid?' do
		it 'returns true' do
			@user_form.validate(params)
			expect(@user_form.valid?).to be_truthy
		end
		it 'returns false' do
			@user_form.validate({})
			expect(@user_form.valid?).to be_falsey
		end
	end

	context '#errors' do
		it 'messages should not be empty' do
			@user_form.validate({})
			expect(@user_form.errors.messages).not_to be_empty
		end
	end

	context '#sync_with_model' do
		it 'should sync params with model' do
			@user_form.validate(params)
			@user_form.sync_with_model
			expect(@user_model.email).to eq(params[:email])
			expect(@user_model.user_profile.first_name).to eq(params[:user_profile][:first_name])
		end
	end

	context 'contract' do
		before(:example) do
			@user_form = UserForm.new(@user_model, contract: true)
			@user_form.validate
		end

		context '#valid?' do
			it 'returns false' do
				expect(@user_form.valid?).to be_falsey
			end
		end

		context '#errors' do
			it 'messages should not be empty' do
				expect(@user_form.errors.messages).not_to be_empty
			end
		end
	end

	context 'custom validations' do
		context '#valid?' do
			it 'return true' do
				@user_form_custom.validate(params)
				expect(@user_form_custom.valid?).to be_truthy
			end
			it 'return false' do
				params[:email] = 'bad-email.com'
				@user_form_custom.validate(params)
				expect(@user_form_custom.valid?).to be_falsey
			end
		end
	end

	context 'long tree' do
		context '#sync_with_model' do
			it 'should sync params with model' do
				@user_form_tree.validate(tree_params)
				@user_form_tree.sync_with_model
				expect(@user_model_tree.profile.passport.number).to eq(params[:user_profile][:user_profile_passport][:number])
			end
		end
	end

	context 'delegation' do
		before(:example) do
			@user_form.validate(params)
		end

		it 'properties' do
			expect(@user_form.email).to be_a(::Frigate::Form::Property)
		end

		it 'associations' do
			expect(@user_form.user_profile).to be_a(::Frigate::Form::Association)
			expect(@user_form.user_profile.first_name).to be_a(::Frigate::Form::Property)
		end
	end

	describe Frigate::Form::Property do
		context '#root' do
			it 'should be instance of Frigate::Form::Base' do
				@user_form.validate(params)
				expect(@user_form.email.root).to be_a(Frigate::Form::Base)
				expect(@user_form.user_profile.first_name.root).to be_a(Frigate::Form::Base)
			end
		end
	end

	describe Frigate::Form::Association do
		context '#root' do
			it 'should be instance of Frigate::Form::Base' do
				@user_form.validate(params)
				expect(@user_form.user_profile.root).to be_a(Frigate::Form::Base)
			end
		end
	end
end